function banded_2Toeplitz_definite_main_routine

disp('This is a program for plotting the numerical range of a 2-Toeplitz operator,')
disp('that is an operator associated to an infinite matrix of the form:')
disp(' ');
disp('a0  a_menos1  a_menos2  a_menos3 ...') 
disp('b1     b0     b_menos1  b_menos2  b_menos3 ...')
disp('a2     a1        a0     a_menos1  a_menos2  a_menos3 ...')
disp('b3     b2        b1        b0     b_menos1  b_menos2  b_menos3 ...')
disp('...   ...       ...       ...       ...        ...      ...    ...') 
disp(' ');

disp('We consider always banded 2-Toeplitz operators with bandwidth M');
disp(' ');

disp('This Matlab program uses N.Bebiano, J. da Providncia and A.Nata algorithm 1 included on the article')
disp('Numerical ranges of banded biperiodic Toeplitz operators, J. Math. Anal. Appl, in press')
disp('Available online 17 August 2012')
disp(' ');

disp('The algorithm is developed performing a reduction to the 2-dimensional case.')
disp('Indeed, the numerical range of a 2-Toeplitz operator is the convex-hull of')
disp('the union of the numerical range of a family of 2 by 2 matrices defined on the interval [0,2pi[.')
disp(' ');

disp('Programmer: Ana Nata')
disp('CMUC and Department of Mathematics') 
disp('Polytechnic Institute of Tomar, Portugal')
disp('e-mail: anata@ipt.pt')
disp(' ');
disp('April 2011')
disp(' ');

disp(' This file is named as banded_2Toeplitz_definite_main_routine.m');
disp('1. run matlab');
disp('2. input this file banded_2Toeplitz_definite_main_routine.m and all the subroutines necessaries that');
disp('2. can be found on http://www.mat-uc.pt/~bebiano');

disp('INPUT:'); 
disp('n = '); %positive integer that gives the number of partions on the interval [0,2pi[
disp('M = '); %positive integer (less or equal 11) that gives the bandwidth M of the 2-Teoplitz operator
disp('enter the elements of the 2-Toeplitz operator according to the bandwidth,')
disp('e.g., in the case of a tridiagonal operator introduce a0= ;b0= ;b1= ;a1= ;a_menos1= ;b_menos1= ')
disp('e.g., n=40; M=3; a0=1; b0=2; b1=1; a1=-1; a_menos1=2; b_menos1=1;')
disp(' ');

%%
i=sqrt(-1);
Tol=10^(-5); 
%n=input('enter a positive integer n= ');
n=40;
n=round(n);
if n<30  
    n=30; 
end 

%M=input('enter the bandwidth M of the 2-Teoplitz operator M= and the elements of the 2-Toeplitz operator');

%EXAMPLE 1 (tridiagonal operator,i.e., M=3)
M=3;
a0=3;b0=-1;b1=2;a1=-4;a_menos1=4;b_menos1=-8; 

% EXAMPLE 2 (pentadiagonal operator, i.e., M=5)
%M=5;
%a0=0;b0=0;b1=1;a1=0;a2=0;b2=1;a_menos1=1;b_menos1=-1;a_menos2=0;b_menos2=0; 

% EXAMPLE 3 (heptadiagonal operator, i.e., M=7)
% M=7;
% b1=1;b3=-1;a_menos1=1;a_menos3=-2;a0=0;b0=0;a1=0;a2=0;b2=0;a3=0;b_menos1=0;a_menos2=0;b_menos2=0;b_menos3=0;

% EXAMPLE 4 (Eneadiagonal operator, i.e., M=9)
% M=9;
% a0=0;b0=0;b1=-1;a1=0;a2=0;b2=0;b3=0;a3=0;a4=0;b4=0;a_menos1=2;b_menos1=0;a_menos2=0;b_menos2=0;a_menos3=0;b_menos3=0;a_menos4=2;b_menos4=-2;

%% Check if M is odd
if mod(M,2)==0
    error('The bandwidth is not valid because it has to be an odd number');
end

if M~=3 && M~=5 && M~=7 && M~=9 && M~=11
    error('Bandwidth not valid because the program is implemented only for bandwidth 3, 5, 7, 9 or 11');
end

%%
X=[];
phi=2*pi/n:2*pi/n:2*pi;
    for j=1:n; %symbol matrix, i.e., reduction  to a family pf 2 by 2 matrices
        if M==3 %tridiagonal (i.e., bandwidth M=3)
            A=[a0 a_menos1+a1*(cos(phi(j))+i*sin(phi(j)));b1+b_menos1*(cos(phi(j))-i*sin(phi(j))) b0];
        elseif M==5 %pentadiagonal (i.e., bandwidth M=5)
            A=[a_menos2*(cos(phi(j))-i*sin(phi(j)))+a0+a2*(cos(phi(j))+i*sin(phi(j))) a_menos1+a1*(cos(phi(j))+i*sin(phi(j))); b1+b_menos1*(cos(phi(j))-i*sin(phi(j))) b_menos2*(cos(phi(j))-i*sin(phi(j)))+b0+b2*(cos(phi(j))+i*sin(phi(j)))];
        elseif M==7 %Hhptadiagonal (i.e., bandwidth M=7) 
            A=[a_menos2*(cos(phi(j))-i*sin(phi(j)))+a0+a2*(cos(phi(j))+i*sin(phi(j))) a_menos3*(cos(phi(j))-i*sin(phi(j)))+a_menos1+a1*(cos(phi(j))+i*sin(phi(j)))+a3*(cos(2*phi(j))+i*sin(2*phi(j))); b_menos3*(cos(2*phi(j))-i*sin(2*phi(j)))+b_menos1*(cos(phi(j))-i*sin(phi(j)))+b1+b3*(cos(phi(j))+i*sin(phi(j))) b_menos2*(cos(phi(j))-i*sin(phi(j)))+b0+b2*(cos(phi(j))+i*sin(phi(j)))];
        elseif M==9 %Eneadiagonal (i.e., bandwidth M=9)
            A=[a_menos4*(cos(2*phi(j))-i*sin(2*phi(j)))+a_menos2*(cos(phi(j))-i*sin(phi(j)))+a0+a2*(cos(phi(j))+i*sin(phi(j)))+a4*(cos(2*phi(j))+i*sin(2*phi(j))) a_menos3*(cos(phi(j))-i*sin(phi(j)))+a_menos1+a1*(cos(phi(j))+i*sin(phi(j)))+a3*(cos(2*phi(j))+i*sin(2*phi(j))); b_menos3*(cos(2*phi(j))-i*sin(2*phi(j)))+b_menos1*(cos(phi(j))-i*sin(phi(j)))+b1+b3*(cos(phi(j))+i*sin(phi(j))) b_menos4*(cos(2*phi(j))-i*sin(2*phi(j)))+b_menos2*(cos(phi(j))-i*sin(phi(j)))+b0+b2*(cos(phi(j))+i*sin(phi(j)))+b4*(cos(2*phi(j))+i*sin(2*phi(j)))];
        elseif M==11; %% Hendecadiagonal (i.e., bandwidth M=11)
            A=[a_menos4*(cos(2*phi(j))-i*sin(2*phi(j)))+a_menos2*(cos(phi(j))-i*sin(phi(j)))+a0+a2*(cos(phi(j))+i*sin(phi(j)))+a4*(cos(2*phi(j))+i*sin(2*phi(j))) a_menos5*(cos(2*phi(j))-i*sin(2*phi(j)))+a_menos3*(cos(phi(j))-i*sin(phi(j)))+a_menos1+a1*(cos(phi(j))+i*sin(phi(j)))+a3*(cos(2*phi(j))+i*sin(2*phi(j)))+a5*(cos(3*phi(j))+i*sin(3*phi(j))); b_menos5*(cos(3*phi(j))-i*sin(3*phi(j)))+b_menos3*(cos(2*phi(j))-i*sin(2*phi(j)))+b_menos1*(cos(phi(j))-i*sin(phi(j)))+b1+b3*(cos(phi(j))+i*sin(phi(j)))+b5*(cos(2*phi(j))+i*sin(2*phi(j))) b_menos4*(cos(2*phi(j))-i*sin(2*phi(j)))+b_menos2*(cos(phi(j))-i*sin(phi(j)))+b0+b2*(cos(phi(j))+i*sin(phi(j)))+b4*(cos(2*phi(j))+i*sin(2*phi(j)))];
        end
        [X_round_dif]=numerical_range(A,Tol);
        X=[X X_round_dif];
        hold on;
        figure(1)
    end %j

[X_round]=rounding(X); %Rounding X
[X_round_dif]=different_elements(X_round,Tol); %Remove any elements of repeated of X

%Representing the convex hull of the numerical range os the symbol matrices A considering phi=2*pi/n:2*pi/n:2*pi; 
    if size(X_round_dif,2)==1 %The 2-Toeplitz operartor is scalar and hence its numerical range is a point
        plot(real(X_round_dif),imag(X_round_dif),'.blue','MarkerSize',5);
    else
        [col]=collinear(X_round_dif,Tol);
            if col==1 %means that the numerical range of the 2-Toeplitz operator is a line segment
                plot(real(X_round_dif),imag(X_round_dif),'color','blue');
            else
                Ind=convhull(real(X),imag(X));
                plot(real(X(Ind)),imag(X(Ind)),'color','red','linewidth',1.5);
                hold on
            end
    end

xlabel('Real axis');
ylabel('Imaginary axis');
axis equal;
        
